#include<avr/io.h>
#include<util/delay.h>
#include<avr/pgmspace.h>
#include<stdlib.h>
#include"lcd.h"
#include"adc.h"
#include"defines.h"
#include"font5x7.h"
#include"keypad.xbm"


/*
LCD-Routines
*/

//Write to LCD
void lcd_write(char CMD,char DATA)
{
	LCD_RW=0;
	LCD_RS=1;

	LCD_DATA_PORT=CMD;
	_delay_us(3);
	LCD_E=1;
	_delay_us(10);
	LCD_E=0;
	_delay_us(5);
	LCD_RS=0;
	LCD_DATA_PORT=DATA;
	_delay_us(5);
	LCD_E=1;
	_delay_us(10);
	LCD_E=0;
	_delay_us(5);
	LCD_RS=0;
}

//Initialize LCD
void lcd_init(int mode)
{
	LCD_RS=0;
	LCD_RW=0;
	LCD_E=0;
	LCD_CON_DDR=0x07;
	LCD_DATA_DDR=0xff;

	if(mode==1){
		lcd_write(LCD_CMD_MODE,0x32);
		lcd_write(LCD_CMD_CHAR_PITCH,LCD_CHAR_PITCH_HP_8);
	}
	else
	{
		lcd_write(LCD_CMD_MODE,0x3c);
		lcd_write(LCD_CMD_CHAR_PITCH,LCD_CHAR_PITCH_HP_8|0x70);
	}

	lcd_write(LCD_CMD_NUM_CHARS,18);
	lcd_write(LCD_CMD_TIME_DIVISION,80);
	lcd_write(LCD_CMD_DISPLAY_START_LA,0x00);
	lcd_write(LCD_CMD_DISPLAY_START_HA,0x00);
	lcd_write(LCD_CMD_CURSOR_LA,0x00);
	lcd_write(LCD_CMD_CURSOR_HA,0x00);

}




void lcd_clear()
{
	lcd_write(LCD_CMD_CURSOR_LA,0x00);
	lcd_write(LCD_CMD_CURSOR_HA,0x00);
	int i;
	for(i=0;i<1601;i++)
	{
		lcd_write(LCD_CMD_WRITE_DATA,0x00);
	}
	lcd_write(LCD_CMD_CURSOR_LA,0x00);
	lcd_write(LCD_CMD_CURSOR_HA,0x00);
}

void lcd_clear_rect(int x,int y,int w, int h)
{
	int tx,ty;
	w=(w/8);
	x=(x/8);
	for(ty=0;ty<h;ty++)
	{
		int adr=(y*20)+x;
		lcd_write(LCD_CMD_CURSOR_LA,adr&0xff);
		lcd_write(LCD_CMD_CURSOR_HA,(adr>>8)&0xff);
		for(tx=0;tx<w;tx++)
		{
			lcd_write(LCD_CMD_WRITE_DATA,0x00);
		}
		y++;
	}
}

void lcd_goto(int x, int y)
{
	int adr=(y*20)+(x/8);
	lcd_write(LCD_CMD_CURSOR_LA,adr&0xff);
	lcd_write(LCD_CMD_CURSOR_HA,(adr>>8)&0xff);
}

void lcd_pset(int x,int y)
{
	lcd_goto(x,y);
	lcd_write(LCD_CMD_SET_BIT,x&0x07);
}

void lcd_pclear(int x,int y)
{
	lcd_goto(x,y);
	lcd_write(LCD_CMD_CLEAR_BIT,x&0x07);
}

void lcd_xbm(int x,int y,int w,int h,unsigned char bits[],int inverted)
{
	int count;
	int cx,cy;


	count=0;

	for(cy=y;cy<y+h;cy++){
		lcd_goto(x,cy);
		for(cx=0;cx<(w/8);cx++)
		{
			if(inverted){
				lcd_write(LCD_CMD_WRITE_DATA,0xff^pgm_read_byte(bits+(count++)));
			}
			else
			{
				lcd_write(LCD_CMD_WRITE_DATA,pgm_read_byte(bits+(count++)));
			}
		}
	}

}

int abs(int val)
{
	return (val>=0)?val:(val*(-1));
}

//Quelle: Wikipedia "Bresenham's line algorithm"
void line(int x0, int y0, int x1, int y1) {
	int Dx = x1 - x0; 
	int Dy = y1 - y0;
	int tmp;
	int steep = (abs(Dy) >= abs(Dx));
	if (steep) {
		tmp=x0;
		x0=y0;
		y0=tmp;
		tmp=x1;
		x1=y1;
		y1=tmp;
		
		// recompute Dx, Dy after swap
		Dx = x1 - x0;
		Dy = y1 - y0;
	}
	int xstep = 1;
	if (Dx < 0) {
		xstep = -1;
		Dx = -Dx;
	}
	int ystep = 1;
	if (Dy < 0) {
		ystep = -1;		
		Dy = -Dy; 
	}
	int TwoDy = 2*Dy; 
	int TwoDyTwoDx = TwoDy - 2*Dx; // 2*Dy - 2*Dx
	int E = TwoDy - Dx; //2*Dy - Dx
	int y = y0;
	int xDraw, yDraw;
	int x;	
	for (x = x0; x != x1; x += xstep) {		
		if (steep) {			
			xDraw = y;
			yDraw = x;
		} else {			
			xDraw = x;
			yDraw = y;
		}
		// plot
		lcd_pset(xDraw, yDraw);
		// next
		if (E > 0) {
			E += TwoDyTwoDx; //E += 2*Dy - 2*Dx;
			y = y + ystep;
		} else {
			E += TwoDy; //E += 2*Dy;
		}
	}
}

//Quelle: www.frozeneskimo.com/samsunglcd/
void draw_char(unsigned short x, unsigned short y, char character) {
	unsigned char fontIndex, i, j;

	/* The 5x7 character set starts at the '!' character (ASCII index 
	* number 33) so we subtract 32 from the ASCII character to find the 
	* index in the 5x7 font table. */	
	fontIndex = character-32;
	/* If the index is out of bounds, bail out */
	if (fontIndex > 94)
	return;
	
	for (i = 0; i < FONT_WIDTH; i++) {
		for (j = 0; j < FONT_HEIGHT; j++) {
			/* Check if the bit/pixel is set, paint accoringly to 
			* the screen */
			if (Font5x7[FONT_WIDTH*fontIndex+i] & (1<<j))
			lcd_pset(x, y+j);
			else	
			lcd_pclear(x, y+j);
		}
		/* Move the LCD cursor through the font width as well */
		x++;
	}
	for(j = 0; j < FONT_HEIGHT; j++){lcd_pclear(x,y+j);}
}

void lcd_puts(int x,int y,char *text)
{
	int xt=x;
	while(*text)
	{
		draw_char(xt,y,(char)*text);
		text++;
		xt+=(FONT_WIDTH+1);
	}

}

char buffer[32];
static char map[4][3]={{'1','2','3'},{'4','5','6'},{'7','8','9'},{'C','0','K'}};

int lcd_getvalue()
{
	int ready=0;
	char value[32];
	int vpos=0;
	int value_i=0;
	int x,y;
	int col,row;

	for(vpos=0;vpos<32;vpos++){value[vpos]=0;}
	vpos=0;
	lcd_clear();
	lcd_xbm(0,0,keypad_width,keypad_height,keypad_bits,0);

	while(!ready){

		if(touched()){
			x=read_xval();
			y=read_yval();
			
			col=-1;
			if(x>98)col=0;
			if(x>117)col=1;
			if((x>135)&(x<160))col=2;
			row=-1;
			if(y>1)row=0;
			if(y>19)row=1;
			if(y>38)row=2;
			if((y>57)&(y<79))row=3;

			if((col>=0)&(row>=0))
			{
				value[vpos]=map[row][col];
				if(value[vpos]=='C'){for(vpos=0;vpos<32;vpos++){value[vpos]=0;} vpos=-1;}
				if(value[vpos]=='K'){value_i=atoi(value);vpos=-1;ready=1;}
				if(vpos>10)vpos--;
				vpos++;
				value[vpos]=0;
				
				lcd_clear_rect(8,32,72,10);
				lcd_puts(12,33,value);

			}

			while(touched()){}
		}
	}

	lcd_clear();
	return value_i;
}
